
# q-expansion of newform 441.8.a.g, downloaded from the LMFDB on 01 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 441
weight = 8
poly_data = [-690, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 18], 1]]

hecke_ring_character_values = None
aps_data = [[-10, 0], [0, 0], [0, -1], [0, 0], [4748, 0], [0, 7], [0, 74], [0, -55], [75520, 0], [109366, 0], [0, 10], [199650, 0], [0, -1050], [-419340, 0], [0, -470], [466930, 0], [0, 655], [0, 365], [994180, 0], [998912, 0], [0, -4736], [-2832904, 0], [0, 19411], [0, -12160], [0, 22442], [0, -20765], [0, 23374], [5429540, 0], [3057066, 0], [-21947690, 0], [-9893000, 0], [0, -9885], [53454970, 0], [0, -31045], [-7328254, 0], [20105328, 0], [0, 57459], [-49418660, 0], [0, -127386], [0, 111705], [-49984996, 0], [0, -318605], [149749912, 0], [87989130, 0], [-10659070, 0], [0, -174990], [196146708, 0], [0, 600852], [0, 393803], [0, 202455], [323477690, 0], [-245597704, 0], [0, -732750], [0, 537845], [0, -439552], [393366640, 0], [0, 568285], [0, -1031680], [85964030, 0], [669542858, 0], [0, -380335], [0, 279839], [0, -719803], [0, -2029800], [0, -1083886], [-413425190, 0], [-144181692, 0], [245621910, 0], [1171269500, 0], [0, 149135], [0, 2066348], [-1001666816, 0], [0, 2049540], [-452025410, 0], [-1243150284, 0], [0, -3297806], [-1093534594, 0], [0, 2879339], [-959535398, 0], [0, 2249810], [0, 1763545], [-692069138, 0], [811538968, 0], [0, 3550218], [0, -5063680], [1443659060, 0], [917900926, 0], [-132644670, 0], [0, -2493155], [3732664280, 0], [0, -3514981], [0, 5696590], [-2133185280, 0], [1873281268, 0], [-105598476, 0], [0, 2940532], [0, -9416275], [0, 7609690], [0, -3466567], [2830005398, 0], [5448680540, 0], [-5746517110, 0], [0, -16300455], [3591881906, 0], [-5193004028, 0], [0, -7805060], [0, -10953999], [0, 9604244], [8277173536, 0], [0, 2137240], [0, -2907176], [7438256530, 0], [3651722450, 0], [0, -14594395], [3309005968, 0], [2993109002, 0], [0, 18826367], [0, -11556338], [7360248550, 0], [-4404152636, 0], [0, 23644085], [6092170590, 0], [0, -20246563], [-9178165820, 0], [0, -5233395], [15671499478, 0], [-16662300414, 0], [0, -24439570], [0, 32756766], [0, -9218831], [30658556, 0], [4110167920, 0], [1080350232, 0], [-18939392190, 0], [0, 39516770], [0, -37966530], [0, 2796819], [0, -10239835], [0, -23350075], [-6555967234, 0], [0, 4980325], [9697800578, 0], [28173595280, 0], [-17021720060, 0], [0, 22449885], [0, 67646530], [0, 11835103], [0, 22160478], [0, -10619265], [-3633540200, 0], [19126337450, 0], [0, 39200000], [20023767780, 0], [0, 18296354], [18164396220, 0], [36126995672, 0], [14728782624, 0], [0, -39016230], [0, 46898712], [0, 54967875], [8851990340, 0], [25613543030, 0], [3333488720, 0], [0, -53164665], [20518230770, 0], [0, -18197462], [-36418482312, 0], [0, 71207853], [-9476140966, 0], [0, 44697813], [-16584936716, 0], [0, 55504575], [63982387088, 0], [-60191057530, 0], [0, 97773210], [0, 76111420], [32683302332, 0], [-57316701202, 0], [0, 119812196], [0, -123984055], [32166487720, 0], [0, -153919395], [35187003490, 0], [0, 133804796], [24111256360, 0], [0, 64766965], [9925195110, 0], [0, -58101607], [3436138534, 0], [0, -90210830], [0, 125532380], [24431358740, 0], [-82258164972, 0], [0, 176534945], [-15516266260, 0], [0, -23060404], [783228822, 0], [-34461231830, 0], [0, 16198056], [0, 234345264], [-67565169334, 0], [0, 124427590], [0, 62037305], [0, 18886920], [0, 1465695], [0, 123453159], [0, 172349710], [-36941938540, 0], [-91353405586, 0], [0, 152425255], [81160384070, 0], [0, 160565195], [-123235455600, 0], [0, -23654345], [0, -106692400], [0, -169780610], [68544869160, 0], [0, 71057690], [90150428960, 0], [-50969936810, 0], [2219340882, 0], [0, -98774480], [151097522174, 0], [-111451619640, 0], [0, 34207999], [193026545154, 0], [0, 78729552], [-26934002744, 0], [0, 406477212], [64554568348, 0], [73963235610, 0], [0, 356077515], [94805877672, 0], [-147709009082, 0], [0, 346690197], [0, 271093848], [0, -145314210], [-149227285630, 0], [-21750295324, 0], [0, 549112060], [181196075860, 0], [0, 313562145], [0, -277911372], [229336956726, 0], [0, 352891000], [0, -418647090], [0, 605298176], [0, 124346135], [-275047866284, 0], [19297943320, 0], [-59933943690, 0], [0, 51864700], [-85220053840, 0], [0, 194311040], [0, 439084643], [220568259244, 0], [92615506562, 0], [0, 492532351], [0, 607303907], [0, -508146358], [307554515560, 0], [-99150279020, 0], [0, 22198165], [0, -30108183], [0, -57664386], [0, -649678665], [-173831876486, 0], [0, -671869800], [-128818740020, 0], [33103145630, 0], [0, 314446735], [-246864406980, 0], [0, -805897460], [-291515636136, 0], [0, -244252792], [0, 163593844], [-420984683780, 0], [141727668486, 0], [-7560396078, 0], [0, -426233585], [0, 434720276], [292395861392, 0], [0, -30161502], [0, 498266545], [0, -787723803], [-126682786072, 0], [-52610737230, 0], [-296179340270, 0], [0, 67520750], [0, 57313620], [247794587882, 0], [0, -565307361], [-199159301470, 0], [0, -946017065], [255657090950, 0], [0, -1071685275], [0, 923031040], [0, -152236539], [0, -640509815], [0, -1166091941], [0, 51703562], [415963808950, 0], [-60789435464, 0], [-356448381340, 0], [168722980772, 0], [-461982265890, 0], [-499749885220, 0], [0, -1060403575], [-364951815056, 0], [-479077849170, 0], [0, -637347460], [-52117083506, 0], [295031346082, 0], [-357047506020, 0], [179587580240, 0], [0, 1275653760], [0, 668662505], [442089754472, 0], [0, -219986256], [106524365554, 0], [0, -902664165], [308831315670, 0], [0, 641092095], [-243078140520, 0], [-739178554210, 0], [0, -1685096230], [523689866444, 0], [0, -191121727], [-738732590200, 0], [-510721078450, 0], [-71311578762, 0], [-549562829450, 0], [0, 1849470350], [0, 241205241], [-273245233732, 0], [0, -157944353], [-597108711526, 0], [0, -109581732], [0, 149433060], [0, 577707782], [160463153870, 0], [-56096964590, 0], [0, -991139835], [92622545472, 0], [1095199066070, 0], [166941680036, 0], [0, -154881745], [482127475140, 0], [0, 1719066020], [0, 39755659], [0, -615616945], [-801923279110, 0], [422745041962, 0], [0, -196920602], [-654770748146, 0], [0, -211603140], [0, 1533106630], [0, -822377125], [-264029652050, 0], [-477671395280, 0], [-1304840356370, 0], [0, 1622854120], [695704925960, 0], [437699492836, 0], [0, 1241253961], [878870821230, 0], [0, -872512879], [-166363222160, 0], [990817835058, 0], [863396397068, 0], [0, -1825575205], [367024233160, 0], [-1520808532574, 0], [0, 2009302760], [1340209482230, 0], [0, -3382556985], [-1280954219848, 0], [0, 339690928], [0, -718577770], [0, -3086409270], [0, 2579788145], [695466594590, 0], [-1499194734000, 0], [-103028609570, 0], [0, -1921136525], [0, -1951372496], [1423692490008, 0], [0, 448217465], [-1691313689860, 0], [0, 1525217050], [-1037162802394, 0], [0, 4175496555], [1087998967204, 0], [0, -3587412037], [-450064470208, 0], [-60254938410, 0], [0, -3616828310], [0, 3469759930], [1663823271532, 0], [-1506621828962, 0], [0, 2683258425], [-46730852290, 0], [790779484120, 0], [0, -3423577018], [0, 4528416785], [0, 3202681630], [547800840550, 0], [1323182344678, 0], [0, -572655129], [0, 2996438185], [0, 497801592], [-208220855650, 0], [2008303768700, 0], [-1550525530572, 0], [-632589115450, 0], [0, 1336722505], [1339262437496, 0], [0, -1209851458], [0, 1868043534], [0, 205342612], [383722070566, 0], [0, 4463719205], [-934452333320, 0], [0, 962156405], [0, -3035491438], [0, -4032744369], [-1378766999900, 0], [1313572660874, 0], [0, 461502835], [0, 3977590110]]
